package jmslexamples;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.IOException;

import javax.swing.JFrame;

import com.didkovsky.portview.PVButton;
import com.didkovsky.portview.PVLabel;
import com.didkovsky.portview.PVPanel;
import com.didkovsky.portview.PVScrollbar;
import com.didkovsky.portview.PVScrollbarListener;
import com.didkovsky.portview.swing.ViewFactorySwing;
import com.softsynth.jmsl.Composable;
import com.softsynth.jmsl.JMSL;
import com.softsynth.jmsl.JMSLMixerContainer;
import com.softsynth.jmsl.JMSLRandom;
import com.softsynth.jmsl.MusicShape;
import com.softsynth.jmsl.ParallelCollection;
import com.softsynth.jmsl.Playable;
import com.softsynth.jmsl.jsyn2.JSynMusicDevice;
import com.softsynth.jmsl.jsyn2.JSynUnitVoiceInstrument;
import com.softsynth.jmsl.midi.MidiIOFactory;
import com.softsynth.jmsl.midi.MidiIO_JavaSound;
import com.softsynth.jmsl.midi.MidiInstrument;
import com.softsynth.jmsl.util.TuningET;
import com.softsynth.jmsl.view.PVLabelAdapter;
import com.softsynth.jmsl.view.PVPanelAdapter;

/**
 * Control the JMSL clock rate for a ParallelCollection of MusicShapes by moving
 * a scrollbar.
 *
 * Each of four musicshapes plays elements that are twice as fast and an octave
 * higher than the previous (gamelan style). This lets use clearly hear that the
 * elements stay in synch when the JMSL clock rate changes . Thanks to Phil Burk for this new 
 * feature. The global JMSL clock rate can now be smoothly varied.
 *
 * 2023-01-17
 *
 * @author Nick Didkovsky, 2023-01-17, copyright (c) 2023, Nick Didkovsky
 */
public class NicksDreamMIDI extends JFrame implements ActionListener, PVScrollbarListener {

    private JMSLMixerContainer mixer;
    private PVButton startButton;
    private PVButton stopButton;
    private PVScrollbar speedyScrollbar;
    private PVLabel speedyLabel;

    private ParallelCollection parCol;

    private double currentClockRate = 1.0;
    private int numRepeats = 1000;

    private void buildLayout() {
        setLayout(new BorderLayout(20, 20));

        PVPanel p = new PVPanelAdapter();
        p.setLayout(new GridLayout(0, 1));
        speedyLabel = new PVLabelAdapter(durString());

        speedyScrollbar = new PVScrollbar((int) currentClockRate * 1000, 10, 6000);
        speedyScrollbar.addPVScrollbarListener(this);
        // set the size of the scrollbar
        speedyScrollbar.setSize(320, 25);
        // set the increment that the value will jump when user clicks inside
        // scrollbar
        speedyScrollbar.setPageIncrement(100);

        p.add(speedyLabel.getComponent());
        p.add(speedyScrollbar.getComponent());
        add(BorderLayout.CENTER, p.getComponent());

        p = new PVPanelAdapter();
        p.add((startButton = JMSL.getViewFactory().createButton("Start")).getComponent());
        p.add((stopButton = JMSL.getViewFactory().createButton("Stop")).getComponent());
        add(BorderLayout.SOUTH, p.getComponent());

        add(BorderLayout.NORTH, mixer.getPanAmpControlPanel());

        startButton.addActionListener(this);
        stopButton.addActionListener(this);
    }

    String LOCATION_OF_SOUNDFONT = "F:/jwork/SoundFonts/FluidR3/FluidR3 GM2-2.SF2";

    private void initJavaSoundMidi() {
        try {
            JMSL.midi = new MidiIOFactory().getMidiIO(MidiIOFactory.MidiIO_JAVASOUND);

            // Gervill only available after Java 1.7

            String javaSpecStr = System.getProperty("java.specification.version");
            double javaSpecVersion = javaSpecStr != null ? Double.parseDouble(javaSpecStr) : 0;
            String requestedDeviceName = javaSpecVersion < 1.7 ? "Java Sound Synthesizer" : "Gervill";

            // you could change this to a soundbank.gm location if pre-Gervill, or just
            // leave it and
            // JavaSound will ignore
            ((MidiIO_JavaSound) JMSL.midi).setSoundbankFile(new File(LOCATION_OF_SOUNDFONT));
            JMSL.midi.setOutDevice(requestedDeviceName);

            JMSL.midi.open();
        } catch (IOException e2) {
            e2.printStackTrace();
        }
    }

    int NUM_MUSICSHAPES = 4;

    void buildHierarchy() {
        parCol = new ParallelCollection();
        parCol.addStartPlayable(new Playable() {
            @Override
            public double play(double time, Composable parent) throws InterruptedException {
                System.out.println("Parallel Collection starts at time=" + time);
                return time;
            }
        });
        parCol.addRepeatPlayable(new Playable() {
            @Override
            public double play(double time, Composable parent) throws InterruptedException {
                System.out.println("Parallel Collection repeats at time=" + time);
                return time;
            }
        });
        for (int i = 0; i < NUM_MUSICSHAPES; i++) {
            MidiInstrument ins  = new MidiInstrument(1 + i);
           
            MusicShape s = new MusicShape(ins.getDimensionNameSpace());
            s.setInstrument(ins);

            s.addStartPlayable(new Playable() {
                @Override
                public double play(double time, Composable parent) throws InterruptedException {
                    System.out.println("MusicShape " + parent.getName() + " starts at time=" + time);
                    return time;
                }
            });
            s.addStopPlayable(new Playable() {
                @Override
                public double play(double time, Composable parent) throws InterruptedException {
                    System.out.println(
                            "MusicShape stops at time=" + time + ", at " + System.currentTimeMillis() + " msec");
                    return time;
                }
            });

            double duration = Math.pow(2, i) * 0.25;
            double totalDur = 4;
            int numElements = (int) (totalDur / duration);
            System.out.println("duration = " + duration + ", num elements=" + numElements);

            
            for (int j = 0; j < numElements; j++) {
                double[] data = s.getDefaultArray();
                data[0] = duration;
                data[1] = 96 - i * 12 + JMSLRandom.choose(12);
                data[2] = 65;
                data[3] = duration * 0.5;
                s.add(data);
            }
//			s.print();
            parCol.add(s);
        }
        JMSL.clock.setRate(currentClockRate);
        parCol.setRepeats(numRepeats);
    }

    String durString() {
        return "Clock Rate: " + currentClockRate;
    }

    private void buildMixer() {
        mixer = new JMSLMixerContainer();
        mixer.start();
        double[] pans = { 0, 0.33, 0.66, 1.0 };
        double[] amps = { 0.4, 0.4, 0.5, 0.5 };
        for (int i = 0; i < parCol.size(); i++) {
            double pan = pans[i];
            double amp = amps[i];
            mixer.addInstrument(((MusicShape) parCol.get(i)).getInstrument(), pan, amp);
        }
    }

    public void build() {
        JMSL.setViewFactory(new ViewFactorySwing());

        JMSL.clock.setAdvance(0.1);
        JMSLRandom.randomize(); 
        initJavaSoundMidi();
        buildHierarchy();
        buildMixer();
        buildLayout();

        System.out.println("JMSL clock is a " + JMSL.clock.getClass().getName());

    }

    public void cleanup() {
        removeAll();
        parCol.finishAll();
        try {
            parCol.waitForDone();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        JMSL.closeMusicDevices();
    }

    /**
     * the dream is to move this scrollbar and have the entire parallel collection
     * smoothly follow the timestretch change, staying in synch.
     */
    void handleSpeedyChange() {
        currentClockRate = speedyScrollbar.getValue() / 1000.0;
        speedyLabel.setText(durString());
        JMSL.clock.setRate(currentClockRate);
    }

    void handleStart() {
        parCol.finishAll();
        try {
            parCol.waitForDone();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        parCol.launch(JMSL.now());
    }

    void handleStop() {
        parCol.finishAll();
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        Object source = e.getSource();
        if (source == startButton)
            handleStart();
        if (source == stopButton)
            handleStop();
    }

    public static void main(String args[]) {
        NicksDreamMIDI nicksDream = new NicksDreamMIDI();
        nicksDream.build();
        nicksDream.pack();
        nicksDream.setVisible(true);
        nicksDream.addWindowListener(new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent e) {
                nicksDream.cleanup();
                System.exit(0);
            }
        });
    }

    @Override
    public void notifyScrollbarValueChanged(PVScrollbar sb) {
        if (sb == speedyScrollbar) {
            handleSpeedyChange();
        }
    }
}
