package jmslexamples.jsyn2;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JFrame;

import com.didkovsky.portview.PVButton;
import com.didkovsky.portview.PVLabel;
import com.didkovsky.portview.PVPanel;
import com.didkovsky.portview.PVScrollbar;
import com.didkovsky.portview.PVScrollbarListener;
import com.didkovsky.portview.swing.ViewFactorySwing;
import com.softsynth.jmsl.Composable;
import com.softsynth.jmsl.JMSL;
import com.softsynth.jmsl.JMSLMixerContainer;
import com.softsynth.jmsl.JMSLRandom;
import com.softsynth.jmsl.MusicShape;
import com.softsynth.jmsl.ParallelCollection;
import com.softsynth.jmsl.Playable;
import com.softsynth.jmsl.jsyn2.JSynMusicDevice;
import com.softsynth.jmsl.jsyn2.JSynUnitVoiceInstrument;
import com.softsynth.jmsl.util.TuningET;
import com.softsynth.jmsl.view.PVLabelAdapter;
import com.softsynth.jmsl.view.PVPanelAdapter;

/**
 * Demonstrates why setTimeStretch() is *NOT* the way to smoothly speed up and slow down a collection's playback!
 * See NicksDreamClock to see the right way to do this!
 * 
 * Control the timestretch of a ParallelCollection of MusicShapes by moving a
 * scrollbar.
 * 
 * Each of four musicshapes plays elements that are twice as fast and an octave
 * higher than the previous (gamelan style). This lets use clearly hear that the
 * elements get out of sync when the timestretch changes, then get back in sync
 * on the next repeat of the parallel collection.
 * 
 * Does *not* work as desired, since timestretch of musicshapes is not
 * interruptable. The longer durations of elements have to complete before the
 * MusicShape exhibits the change, so the elements of MusicShapes with shorter
 * durations get out of sync with MusicShapes whose elements have longer
 * durations.
 * 
 * 2023-01-17
 * 
 * @author Nick Didkovsky, 2023-01-17, copyright (c) 2023, Nick Didkovsky
 */
public class NicksDreamWrongWay extends JFrame implements ActionListener, PVScrollbarListener {

	private JMSLMixerContainer mixer;
	private PVButton startButton;
	private PVButton stopButton;
	private PVScrollbar speedyScrollbar;
	private PVLabel speedyLabel;

	private ParallelCollection parCol;

	private double currentTimeStretch = 1.0;
	private int numRepeats = 1000;

	private void buildLayout() {
		setLayout(new BorderLayout(20, 20));

		PVPanel p = new PVPanelAdapter();
		p.setLayout(new GridLayout(0, 1));
		speedyLabel = new PVLabelAdapter(durString());

		speedyScrollbar = new PVScrollbar((int) currentTimeStretch * 1000, 10, 6000);
		speedyScrollbar.addPVScrollbarListener(this);
		// set the size of the scrollbar
		speedyScrollbar.setSize(320, 25);
		// set the increment that the value will jump when user clicks inside
		// scrollbar
		speedyScrollbar.setPageIncrement(100);

		p.add(speedyLabel.getComponent());
		p.add(speedyScrollbar.getComponent());
		add(BorderLayout.CENTER, p.getComponent());

		p = new PVPanelAdapter();
		p.add((startButton = JMSL.getViewFactory().createButton("Start")).getComponent());
		p.add((stopButton = JMSL.getViewFactory().createButton("Stop")).getComponent());
		add(BorderLayout.SOUTH, p.getComponent());

		add(BorderLayout.NORTH, mixer.getPanAmpControlPanel());

		startButton.addActionListener(this);
		stopButton.addActionListener(this);
	}

	int NUM_MUSICSHAPES = 4;

	void buildHierarchy() {
		parCol = new ParallelCollection();
		parCol.addStartPlayable(new Playable() {
			public double play(double time, Composable parent) throws InterruptedException {
				System.out.println("Parallel Collection starts at time=" + time);
				return time;
			}
		});
		parCol.addRepeatPlayable(new Playable() {
			public double play(double time, Composable parent) throws InterruptedException {
				System.out.println("Parallel Collection repeats at time=" + time);
				return time;
			}
		});
		for (int i = 0; i < NUM_MUSICSHAPES; i++) {
			JSynUnitVoiceInstrument ins = new JSynUnitVoiceInstrument(1,
					jmslexamples.jsyn2.unitvoices.SpeedyUnitVoice.class.getName());
			MusicShape s = new MusicShape(ins.getDimensionNameSpace());
			s.setInstrument(ins);

			s.addStartPlayable(new Playable() {
				public double play(double time, Composable parent) throws InterruptedException {
					System.out.println("MusicShape " + parent.getName() + " starts at time=" + time);
					return time;
				}
			});

			double duration = Math.pow(2, i) * 0.25;
			double totalDur = 4;
			int numElements = (int) (totalDur / duration);
			System.out.println("duration = " + duration + ", num elements=" + numElements);

			double referenceFrequency = 55;
			int octaveScaler = 4 - i;
			double rootFrequency = referenceFrequency * Math.pow(2, octaveScaler);
			System.out.println(i + ") rootFrequency=" + rootFrequency);
			double frequency = rootFrequency;
			TuningET tuning = (TuningET) ins.getTuning();
			for (int j = 0; j < numElements; j++) {
				double denom = (double) JMSLRandom.choose(5) + 1.0;
				double numer = denom + (double) JMSLRandom.choose(5) + 1.0;
				double ratio = numer / denom;
				frequency *= ratio;
				while (frequency < rootFrequency)
					frequency *= 2;
				while (frequency > rootFrequency * 2)
					frequency /= 2;
				System.out.println("frequency=" + frequency + ", rootFrequency=" + rootFrequency);
				double[] data = s.getDefaultArray();
				data[0] = duration;
				data[1] = tuning.getPitch(frequency);
				data[2] = 0.5;
				data[3] = duration * 0.9;
				s.add(data);
			}
//			s.print();
			parCol.add(s);
		}
		parCol.setTimeStretch(currentTimeStretch);
		parCol.setRepeats(numRepeats);
	}

	String durString() {
		return "Time Stretch: " + currentTimeStretch;
	}

	private void buildMixer() {
		mixer = new JMSLMixerContainer();
		mixer.start();
		double[] pans = { 0, 0.33, 0.66, 1.0 };
		double[] amps = { 0.1, 0.4, 0.5, 0.6 };
		for (int i = 0; i < parCol.size(); i++) {
			double pan = pans[i];
			double amp = amps[i];
			mixer.addInstrument(((MusicShape) parCol.get(i)).getInstrument(), pan, amp);
		}
	}

	public void build() {
		JMSL.setViewFactory(new ViewFactorySwing());

		JMSL.clock.setAdvance(0.1);
		JMSLRandom.randomize();
		JSynMusicDevice.instance().open();

		buildHierarchy();
		buildMixer();
		buildLayout();

	}

	public void cleanup() {
		removeAll();
		parCol.finishAll();
		try {
			parCol.waitForDone();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		JMSL.closeMusicDevices();
	}

	/**
	 * the dream is to move this scrollbar and have the entire parallel collection
	 * smoothly follow the timestretch change, staying in synch.
	 */
	void handleSpeedyChange() {
		currentTimeStretch = speedyScrollbar.getValue() / 1000.0;
		speedyLabel.setText(durString());
		parCol.setTimeStretch(currentTimeStretch);
	}

	void handleStart() {
		parCol.finishAll();
		try {
			parCol.waitForDone();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		parCol.launch(JMSL.now());
	}

	void handleStop() {
		parCol.finishAll();
	}

	public void actionPerformed(ActionEvent e) {
		Object source = e.getSource();
		if (source == startButton)
			handleStart();
		if (source == stopButton)
			handleStop();
	}

	public static void main(String args[]) {
		NicksDreamWrongWay nicksDream = new NicksDreamWrongWay();
		nicksDream.build();
		nicksDream.pack();
		nicksDream.setVisible(true);
		nicksDream.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				nicksDream.cleanup();
				System.exit(0);
			}
		});
	}

	public void notifyScrollbarValueChanged(PVScrollbar sb) {
		if (sb == speedyScrollbar) {
			handleSpeedyChange();
		}
	}
}
