/*
 * Created by Nick Didkovsky on Nov 28, 2004
 *
 */
package jmsltutorial;

import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.FlowLayout;
import java.awt.Label;
import java.awt.Panel;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JFrame;

import com.softsynth.jmsl.Instrument;
import com.softsynth.jmsl.JMSL;
import com.softsynth.jmsl.JMSLMixerContainer;
import com.softsynth.jmsl.JMSLRandom;
import com.softsynth.jmsl.MusicJob;
import com.softsynth.jmsl.MusicShape;
import com.softsynth.jmsl.PlayLurker;
import com.softsynth.jmsl.jsyn2.JSynMusicDevice;
import com.softsynth.jmsl.jsyn2.JSynUnitVoiceInstrument;
import com.softsynth.jmsl.view.JMSLScrollbar;
import com.softsynth.jmsl.view.JMSLScrollbarProcessor;

/**
 * Play a melody with a MusicShape while a PlayLurker is notified of every
 * element played. With some probability, the PlayLurker doubles the pitch
 * played by the MusicShape. This is modelled after Sabbath Bride,
 * http://www.punosmusic.com/pages/sabbathbride/sabbathbride.html
 * 
 * 
 * @author Nick Didkovsky, (c) 2004 All rights reserved, Email:
 *         nick@didkovsky.com
 * 
 */
public class PlayLurkerToot01 extends JFrame implements ActionListener, JMSLScrollbarProcessor {

    JMSLMixerContainer mixer;
    SabbathBrideMusicShape sabbathBrideMusicShape;
    JSynUnitVoiceInstrument ghostVoiceInstrument;
    GhostVoicePlayLurker lurker;
    JMSLScrollbar probScrollbar;
    Label probLabel;

    Button startButton;
    Button stopButton;

    public void start() {
        JMSL.clock.setAdvance(0.1);
        JSynMusicDevice.instance().open();
        mixer = new JMSLMixerContainer();
        mixer.start();

        // this MusicShape comes with its own data and its own custom
        // instrument
        sabbathBrideMusicShape = new SabbathBrideMusicShape();
        sabbathBrideMusicShape.setRepeats(Integer.MAX_VALUE);
        sabbathBrideMusicShape.setTimeStretch(1.2);
        sabbathBrideMusicShape.getInstrument().setName("Melody");

        ghostVoiceInstrument = new JSynUnitVoiceInstrument(32, SlowSineUnitVoice.class.getName());
        ghostVoiceInstrument.setName("Ghost");

        mixer.addInstrument(sabbathBrideMusicShape.getInstrument(), 0.2, 0.1);
        mixer.addInstrument(ghostVoiceInstrument, 0.7, 0.5);

        // IMPORTANT! Add PlayLurker to MusicShape. PlayLurker will be
        // notified of every element played
        lurker = new GhostVoicePlayLurker(ghostVoiceInstrument);
        sabbathBrideMusicShape.addPlayLurker(lurker);

        buildGUI();
    }

    private void buildGUI() {
        Panel buttonPanel = new Panel();
        buttonPanel.setLayout(new FlowLayout());
        buttonPanel.add(startButton = new Button("Start"));
        buttonPanel.add(stopButton = new Button("Stop"));
        stopButton.setEnabled(false);
        startButton.addActionListener(this);
        stopButton.addActionListener(this);

        Panel scrollPanel = new Panel();
        scrollPanel.setLayout(new FlowLayout());
        scrollPanel.add(new Label("Probability of PlayLurker doubling melody"));
        probScrollbar = new JMSLScrollbar(this, 50, 0, 100);
        probScrollbar.setSize(100, 20);
        probScrollbar.setPageIncrement(10);
        probScrollbar.setLineIncrement(1);
        scrollPanel.add(probScrollbar);
        scrollPanel.add(probLabel = new Label("p=" + lurker.getPlayProbability()));

        setLayout(new BorderLayout());
        add(BorderLayout.SOUTH, buttonPanel);
        add(BorderLayout.CENTER, mixer.getPanAmpControlPanel());
        add(BorderLayout.NORTH, scrollPanel);

        pack();
        setVisible(true);
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });
    }

    public void actionPerformed(ActionEvent ev) {
        Object source = ev.getSource();
        if (source == startButton) {
            sabbathBrideMusicShape.launch(JMSL.now());
            startButton.setEnabled(false);
            stopButton.setEnabled(true);
        }
        if (source == stopButton) {
            sabbathBrideMusicShape.finish();
            startButton.setEnabled(true);
            stopButton.setEnabled(false);
        }
    }

    public void JMSLScrollbarValueChanged(JMSLScrollbar jsb) {
        double p = jsb.getValue() / 100.0;
        lurker.setPlayProbability(p);
        probLabel.setText("p=" + p);
    }

    public static void main(String[] args) {
        PlayLurkerToot01 toot = new PlayLurkerToot01();
        toot.start();
    }

}

class GhostVoicePlayLurker implements PlayLurker {

    double playProbability = 0.4;
    Instrument ins;

    public GhostVoicePlayLurker(Instrument ins) {
        this.ins = ins;
    }

    /**
     * PlayLurker is notified of every element being played. With some probability,
     * double the pitch played with a slow sine, 1 or 2 octaves down, at pitch, or 1
     * or 2 octaves up
     * 
     */
    public void notifyPlayLurker(double playTime, MusicJob list, int index) {
        if (JMSLRandom.choose() < getPlayProbability()) {
            double[] data = ((MusicShape) list).get(index);
            double pitch = data[1];
            // build data for instrument to perform
            double[] ghostData = new double[4];
            ghostData[0] = JMSLRandom.choose(5.0, 10.0);
            int transpose = JMSLRandom.choose(5) - 2;
            ghostData[1] = pitch + 12 * transpose;
            ghostData[2] = JMSLRandom.choose(0.25, 0.4);
            ghostData[3] = 0.95 * ghostData[0];

            // now perform it! This plays ONE pitched musical event in synch with the melody
            ins.play(playTime, 1.0, ghostData);
        }
    }

    public double getPlayProbability() {
        return playProbability;
    }

    public void setPlayProbability(double playProbability) {
        this.playProbability = playProbability;
    }
}