/**
 * ParallelCollectionDemo.java
 * 
 * Two jobs stuffed into a ParallelCollection, then launched. Each job has its
 * own timing and repeat count.
 * 
 * @author Phil Burk and Nick Didkovsky
 */
/*
 * (C) 1997 Phil Burk and Nick Didkovsky, All Rights Reserved JMSL is based upon
 * HMSL (C) Phil Burk, Larry Polansky and David Rosenboom.
 */

package jmslexamples;

import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.Panel;
import java.awt.TextArea;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import com.softsynth.jmsl.DefaultMusicClock;
import com.softsynth.jmsl.EventScheduler;
import com.softsynth.jmsl.JMSL;
import com.softsynth.jmsl.JMSLRandom;
import com.softsynth.jmsl.MessagePrinter;
import com.softsynth.jmsl.ParallelCollection;
import com.softsynth.jmsl.view.TextAreaOut;

public class ParallelCollectionDemo extends Frame implements ActionListener {

    TextJob tj1;
    TextJob tj2;
    ParallelCollection col; // a collection to hold both jobs
    DrawingCanvas canvas1; // one for Job 1
    DrawingCanvas canvas2; // one for Job 2
    TextArea textArea;
    Button startButton;
    Button stopButton;

    public void init() {
        JMSLRandom.randomize();
        canvas1 = new DrawingCanvas();
        canvas2 = new DrawingCanvas();
        textArea = new TextArea(20, 80);
        setLayout(new GridLayout(0, 1));
        add(canvas1);
        add(canvas2);

        Panel p = new Panel();
        p.add(startButton = new Button("Start"));
        p.add(stopButton = new Button("Stop"));
        add(p);

        add(textArea);
        buildHierarchy();

        startButton.addActionListener(this);
        stopButton.addActionListener(this);
    }

    void buildHierarchy() {
        // constructor method has a place to draw, a message, and a repeat delay
        tj1 = new TextJob(canvas1, "This job changes color 3 times per second", 0.3333);
        tj1.addRepeatPlayable(new FlasherFunction());
        tj1.setRepeats(12);
        tj2 = new TextJob(canvas2, "This job changes color every second", 1.0);
        tj2.addRepeatPlayable(new FlasherFunction());
        tj2.setRepeats(4);

        JMSL.setSTDOut(new TextAreaOut(textArea));

        col = new ParallelCollection(tj1, tj2);
        col.setRepeats(3);
        col.addStartPlayable(new MessagePrinter("STARTS"));
        col.addRepeatPlayable(new MessagePrinter("REPEATS"));
        col.addStopPlayable(new MessagePrinter("STOPS"));
        col.print();
    }

    public void stop() {
        col.finishAll();
        try {
            col.waitForDone();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }

    void handleStart() {
        col.finishAll();
        col.launch(JMSL.now());
    }

    void handleStop() {
        col.finishAll();
    }

    public void actionPerformed(ActionEvent e) {
        Object source = e.getSource();
        if (source == startButton)
            handleStart();
        if (source == stopButton)
            handleStop();
    }

    /* Can be run as either an application or as an applet. */
    public static void main(String args[]) {
        ParallelCollectionDemo demo = new ParallelCollectionDemo();
        demo.setTitle("Parallel Collection Demo");
        demo.addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent e) {
                System.exit(0);
            }
        });
        demo.init();
        demo.setSize(800, 600);
        demo.setVisible(true);
    }

}